import { map } from 'ramda';

import { Hsl, Hsla, Rgb } from 'types/styles';
import { onColorsHsla } from './variables/theme.styles';

// export const hsla = (h: number, s: number, l: number, a?: number) =>
//   a !== undefined && a < 1 ? `hsla(${h},${s}%,${l}%,${a})` : `hsl(${h},${s}%,${l}%)`;

export const hsla = (hslOrHsla: Hsl | Hsla) => {
  const h = hslOrHsla[0];
  const s = hslOrHsla[1] * 100 + '%';
  const l = hslOrHsla[2] * 100 + '%';

  if (hslOrHsla[3]) {
    const a = hslOrHsla[3] * 100 + '%';
    return `hsla(${h}, ${s}, ${l}, ${a})`;
  }

  return `hsl(${h}, ${s}, ${l})`;
};

/** From: https://github.com/kayellpeee/hsl_rgb_converter/blob/master/converter.js
 * Also see: https://www.rapidtables.com/convert/color/hsl-to-rgb.html
 */
const hslToRgb = (hsl: Hsl): Rgb => {
  const [hue, saturation, lightness] = hsl;
  const chroma = (1 - Math.abs(2 * lightness - 1)) * saturation;
  const huePrime = hue / 60;
  const secondComponent = chroma * (1 - Math.abs((huePrime % 2) - 1));
  const lightnessAdjustment = lightness - chroma / 2;

  let red;
  let green;
  let blue;

  switch (Math.floor(huePrime)) {
    case 0:
      red = chroma;
      green = secondComponent;
      blue = 0;
      break;
    case 1:
      red = secondComponent;
      green = chroma;
      blue = 0;
      break;
    case 2:
      red = 0;
      green = chroma;
      blue = secondComponent;
      break;
    case 3:
      red = 0;
      green = secondComponent;
      blue = chroma;
      break;
    case 4:
      red = secondComponent;
      green = 0;
      blue = chroma;
      break;
    case 5:
    default:
      red = chroma;
      green = 0;
      blue = secondComponent;
  }

  red += lightnessAdjustment;
  green += lightnessAdjustment;
  blue += lightnessAdjustment;

  return [
    Math.round(red * 255),
    Math.round(green * 255),
    Math.round(blue * 255),
  ];
};

/** https://www.w3.org/TR/WCAG20/relative-luminance.xml */
const relativeLuminance = (rgb: Rgb) => {
  const [red, green, blue] = rgb;
  return 0.2126 * red + 0.7152 * green + 0.0722 * blue;
};

/** https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests */
const contrastRatio = (color1: Rgb, color2: Rgb) => {
  const color1Luminance = relativeLuminance(color1) + 0.05;
  const color2Luminance = relativeLuminance(color2) + 0.05;

  const lighterColor = Math.max(color1Luminance, color2Luminance);
  const darkerColor = Math.min(color1Luminance, color2Luminance);

  return lighterColor / darkerColor;
};

/** https://github.com/material-components/material-components-web/blob/master/packages/mdc-theme/_functions.scss */
const colorTone = (hsl: Hsl) => {
  const minimumContrast = 3.1;
  const color = hslToRgb(hsl);

  const lightOnColor = hslToRgb(onColorsHsla.light.regular.slice(0, 3) as Hsl);
  const darkOnColor = hslToRgb(onColorsHsla.dark.regular.slice(0, 3) as Hsl);

  const lightContrast = contrastRatio(color, lightOnColor);
  const darkContrast = contrastRatio(color, darkOnColor);

  return lightContrast < minimumContrast && darkContrast > lightContrast
    ? 'light'
    : 'dark';
};

const contrastTone = (hsl: Hsl) =>
  colorTone(hsl) === 'dark' ? 'light' : 'dark';

export const getColorAndOnColor = (color: Hsl) => ({
  color,
  onColor: onColorsHsla[contrastTone(color)],
});

export const stringifyHslaObject = (object: object) => {
  const stringifyHsla = (value: object | Hsl | Hsla) => {
    if (typeof value === 'object') {
      stringifyHslaObject(value);
    }

    return hsla(value as Hsl | Hsla);
  };

  map(stringifyHsla, object);
};
