import { css } from 'emotion';
import facepaint from 'facepaint';

import { em } from 'styles/_helpers/utils/em.styles';
import {
  breakpoints as majorBreakpoints,
  mediaExpressions as mx,
  minorBreakpoints,
} from './variables.styles';

/**
 * customMq is a method to create complex CSS media queries as strings, based on media
 * expression values stored in arrays.
 *
 * @public
 * @mixin
 * @example
 * // returns '@media screen and (min-width: 20em), (orientation: portrait)'
 * makeMediaQuery(['screen', '(min-width: 20em)'], '(orientation: portrait)')
 * @author Stephen Koo <stephenyrkoo@gmail.com>
 */
export const customMq = (...mediaExpressions: Array<string | string[]>) => {
  const formattedMediaExpressions = [...mediaExpressions].map(
    item => (Array.isArray(item) ? item.join(' and ') : item)
  );

  return '@media ' + formattedMediaExpressions.join(', ');
};

/**
 * mq is a shorthand method to create a min-width CSS media query based on a
 * breakpointName.
 *
 * @public
 * @mixin
 * @author Stephen Koo <stephenyrkoo@gmail.com>
 */
export const mq = (
  breakpointNameOrValue: string | number,
  ...mediaExpressions: string[]
) => {
  const breakpointValue =
    typeof breakpointNameOrValue === 'string'
      ? majorBreakpoints[breakpointNameOrValue] ||
        minorBreakpoints[breakpointNameOrValue]
      : breakpointNameOrValue;

  return customMq([
    mx.minWidth(em(breakpointValue) + 'em'),
    ...mediaExpressions,
  ]);
};

export const mqs = (
  breakpoints = Object.values(majorBreakpoints),
  facepaintOptions = { overlap: true }
) => {
  const breakpointsWithoutZero =
    breakpoints[0] === 0 ? breakpoints.slice(1) : breakpoints;

  return facepaint(
    breakpointsWithoutZero.map(breakpoint => mq(breakpoint)),
    facepaintOptions
  );
};

export const mqsIfMultiple = (
  objectStyleName: string,
  values: number | string | Array<number | string>,
  unitName = ''
) =>
  Array.isArray(values)
    ? mqs()({
        [objectStyleName]: values.map(value => value + unitName),
      })
    : css({
        [objectStyleName]: values + unitName,
      });
