import { css } from 'emotion';

import { mqsIfMultiple } from 'styles/media/utils.styles';
import { mqs } from 'styles/media/utils.styles';
import {
  EnumBreakpoints as bp,
  EnumFontSize,
  FontSizeType,
  TypeElementProperties,
} from 'types/styles';
import { TypographyFoundation } from 'types/styles';
import { round } from 'utils/calculations';
import { typographyFoundation } from './variables.styles';

const getEmLetterSpacing = (tracking: number, fontSizePx: number) => {
  const letterSpacing = round(tracking / fontSizePx);
  return letterSpacing === 0 ? 0 : letterSpacing + 'em';
};

const getRemModularSize = (modularRatio: number, modularStep: number) =>
  round(Math.pow(modularRatio, modularStep));

const getResponsiveLetterSpacingValues = (
  tracking: number,
  baseFontSizePx: TypographyFoundation['baseFontSizePx'],
  fontSize1: number,
  fontSize2?: number
) => [
  getEmLetterSpacing(tracking, fontSize1 * baseFontSizePx[bp.Mobile]),
  getEmLetterSpacing(
    tracking,
    fontSize2
      ? fontSize2 * baseFontSizePx[bp.Tablet]
      : fontSize1 * baseFontSizePx[bp.Tablet]
  ),
];

/**
 * @private
 */
const getResponsiveLetterSpacingStyles = (
  tracking: number,
  fontSizeType: FontSizeType,
  fontSize: number
) => {
  const { modularRatio, baseFontSizePx } = typographyFoundation;

  const responsiveLetterSpacingValues =
    fontSizeType === 'modularScale'
      ? getResponsiveLetterSpacingValues(
          tracking,
          baseFontSizePx,
          getRemModularSize(modularRatio[bp.Mobile], fontSize),
          getRemModularSize(modularRatio[bp.Tablet], fontSize)
        )
      : getResponsiveLetterSpacingValues(tracking, baseFontSizePx, fontSize);

  return mqs()({ letterSpacing: responsiveLetterSpacingValues });
};

export const getTypeElementStyles = (
  typeElementProperties: TypeElementProperties
) => {
  const {
    fontFamily,
    fontSize,
    fontWeight,
    letterSpacing,
    textDecoration,
    textTransform,
    lineHeightPx,
    marginTopPx,
    marginBottomPx,
  } = typeElementProperties;

  const fontSizeCssValue = fontSize
    ? fontSize[0] === 'modularScale'
      ? `var(--ms-${fontSize[EnumFontSize.FontSizeValue]})`
      : fontSize[EnumFontSize.FontSizeValue] +
        fontSize[EnumFontSize.FontSizeType]
    : '';

  const letterSpacingStyles =
    letterSpacing !== undefined
      ? fontSize
        ? getResponsiveLetterSpacingStyles(
            letterSpacing,
            fontSize[EnumFontSize.FontSizeType],
            fontSize[EnumFontSize.FontSizeValue]
          )
        : getResponsiveLetterSpacingStyles(letterSpacing, 'rem', 1)
      : '';

  return css`
    ${fontFamily !== undefined && css({ fontFamily })}
    ${fontSize !== undefined && css({ fontSize: fontSizeCssValue })}
    ${fontWeight !== undefined && css({ fontWeight })}
    ${letterSpacing !== undefined && letterSpacingStyles}
    ${textTransform !== undefined && css({ textTransform })}
    ${textDecoration !== undefined && css({ textDecoration })}
    ${lineHeightPx !== undefined &&
      mqsIfMultiple('lineHeight', lineHeightPx, 'px')}
    ${
      marginTopPx !== undefined
        ? mqsIfMultiple('marginTop', marginTopPx, 'px')
        : css`
            margin-top: 0;
          `
    }
    ${
      marginBottomPx !== undefined
        ? mqsIfMultiple('marginBottom', marginBottomPx, 'px')
        : css`
            margin-bottom: 0;
          `
    }
  `;
};
