import * as CSS from 'csstype';
import { DeepReadonly } from 'utility-types';

import { MsAnimationDuration } from 'styles/animation';

export type FontWeights = Readonly<
  Record<string, CSS.Properties['fontWeight']>
>;

export type FontSizeType = Readonly<'modularScale' | 'rem'>;

export type TypographyFoundation = Readonly<{
  baseFontSizePx: [number, number];
  leadingPx: [number, number];
  modularRatio: [number, number];
  fontFamily: string;
}>;

export const enum EnumFontSize {
  FontSizeType,
  FontSizeValue,
}

export type TypeElementProperties = Readonly<
  Partial<{
    fontFamily: CSS.Properties['fontFamily'];
    fontSize: [FontSizeType, number];
    fontWeight: CSS.Properties['fontWeight'];
    letterSpacing: number;
    textDecoration: CSS.Properties['textDecoration'];
    textTransform: CSS.Properties['textTransform'];
    lineHeightPx: number | number[];
    marginTopPx: number | number[];
    marginBottomPx: number | number[];
  }>
>;

export type TypeProperties = Readonly<Record<string, TypeElementProperties>>;

export type Layout = Readonly<{
  columns: number[];
  gutterSizePx: number[];
  marginSizePx: number[];
  maxGridWidthPx: Array<number | null>;
}>;

export type GridStylesOptions = Readonly<
  Partial<{
    alignItems: string;
    justifyItems: string;
    horizontalGutters: boolean;
    verticalGutters: boolean;
    horizontalMargins: boolean;
    verticalMargins: boolean;
  }>
>;

export const enum EnumBreakpoints {
  Mobile,
  Tablet,
  Desktop,
}

export type Hsl = [number, number, number];
export type Hsla = [number, number, number, number];
export type Rgb = [number, number, number];

export type PaletteHsl = Readonly<
  Partial<{
    0: Hsl;
    50: Hsl;
    100: Hsl;
    200: Hsl;
    300: Hsl;
    400: Hsl;
    500: Hsl;
    600: Hsl;
    700: Hsl;
    800: Hsl;
    900: Hsl;
    1000: Hsl;
    a100: [number, 1, number];
    a200: [number, 1, number];
    a400: [number, 1, number];
    a700: [number, 1, number];
  }>
>;

export type TranslucentPaletteHsla = Readonly<Record<string, Hsla>>;

type OnColorHsla = Readonly<{
  regular: Hsl | Hsla;
  subdued: Hsl | Hsla;
  faint: Hsl | Hsla;
}>;

type OnColor = Readonly<{
  regular: string;
  subdued: string;
  faint: string;
}>;

export type OnColorsHsla = Readonly<{
  light: OnColorHsla;
  dark: OnColorHsla;
}>;

export type ColorAndOnColorHsla = Readonly<{
  color: Hsl | Hsla;
  onColor: OnColorHsla;
}>;

export type ColorAndOnColor = Readonly<{
  color: string;
  onColor: OnColor;
}>;

export type ColorVariantsHsla = Readonly<{
  standard: ColorAndOnColorHsla;
  light?: ColorAndOnColorHsla;
  lightest?: ColorAndOnColorHsla;
  dark?: ColorAndOnColorHsla;
  darkest?: ColorAndOnColorHsla;
}>;

export type BrandColorsHsla = Readonly<{
  primary: ColorVariantsHsla | ColorAndOnColorHsla;
  secondary?: ColorAndOnColorHsla | ColorVariantsHsla;
  additional?:
    | ColorAndOnColorHsla
    | ColorVariantsHsla
    | Record<string, ColorAndOnColorHsla | ColorVariantsHsla>;
}>;

export type BrandColors = Readonly<{
  primary: string;
  secondary?: string;
  additional?: string | Record<string, string>;
}>;

export type ElementColorsHsla = Readonly<{
  background: ColorAndOnColorHsla;
  surface: ColorAndOnColorHsla;
  error: ColorAndOnColorHsla;
  shadow: Hsl | Hsla;
}>;

export type ElementColors = Readonly<{
  background: ColorAndOnColor;
  surface: ColorAndOnColor;
  error: ColorAndOnColor;
  shadow: string;
}>;

export type AnimationTimingFunction = DeepReadonly<
  Partial<{
    easing: {
      standard: string;
      sharp: string;
      accelerate: string;
      decelerate: string;
    };
  }>
>;

export type AnimationType = Readonly<
  Partial<{
    enter: string;
    exitPermanent: string;
    exitTemporary: string;
    standard: string;
  }>
>;

export type ShadowTransition = Readonly<{
  duration: MsAnimationDuration;
  timingFunction: string;
}>;

export type ElevationShadow = [number, number, number, number];

export interface IElevationShadowProperties {
  opacity: number;
  shadows: ElevationShadow[];
}

type Shape = ['sharp' | 'round' | 'cut', number];

export type Theme = Readonly<{
  animation: AnimationType;
  colors: {
    brand: BrandColors;
    element: ElementColors;
  };
  icons: 'filled' | 'sharp' | 'rounded' | 'outlined' | 'two-toned';
  button: {
    shape: Shape;
    heightPx: {
      standard: number;
      dense: number;
    };
    width: {
      minWidthPx: number;
      horizontalPaddingPx: {
        text: number;
        contained: number;
      };
    };
  };
  card: {
    shape: Shape;
  };
  container: {
    shape: Shape;
  };
  dialog: {
    shape: Shape;
  };
  fab: {
    shape: Shape;
  };
  menu: {
    shape: Shape;
  };
  pill: {
    shape: Shape;
  };
  snackbar: {
    shape: Shape;
  };
  tooltip: {
    shape: Shape;
  };
}>;

export interface ITheme {
  theme?: Theme;
}
